@@ -2,10 +2,11 @@
 //!
 //! [`cap_std::fs::Dir`]: https://docs.rs/cap-std/latest/cap_std/fs/struct.Dir.html
 
-use crate::tempfile::LinkableTempfile;
 use cap_std::fs::{Dir, File, Metadata};
-use std::io;
+use std::ffi::OsStr;
 use std::io::Result;
+use std::io::{self, Write};
+use std::ops::Deref;
 use std::path::Path;
 
 /// Extension trait for [`cap_std::fs::Dir`]
@@ -34,46 +35,47 @@ pub trait CapStdExtDirExt {
     /// Remove (delete) a file, but return `Ok(false)` if the file does not exist.
     fn remove_file_optional(&self, path: impl AsRef<Path>) -> Result<bool>;
 
-    /// Create a new anonymous file that can be given a persistent name.
-    /// On Linux, this uses `O_TMPFILE` if possible, otherwise a randomly named
-    /// temporary file is used.  
+    /// Atomically write a file by calling the provided closure.
     ///
-    /// The file can later be linked into place once it has been completely written.
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn new_linkable_file<'p, 'd>(&'d self, path: &'p Path) -> Result<LinkableTempfile<'p, 'd>>;
-
-    /// Atomically write a file, replacing an existing one (if present).
+    /// This uses [`cap_tempfile::TempFile`], which is wrapped in a [`std::io::BufWriter`]
+    /// and passed to the closure.
     ///
-    /// This wraps [`Self::new_linkable_file`] and [`crate::tempfile::LinkableTempfile::replace`].
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn replace_file_with<F, T, E>(
+    /// The closure may also perform other file operations beyond writing, such as changing
+    /// file permissions:
+    ///
+    /// ```rust
+    /// # use std::io;
+    /// # use std::io::Write;
+    /// # fn main() -> io::Result<()> {
+    /// # let somedir = cap_tempfile::tempdir(cap_std::ambient_authority())?;
+    /// use cap_std_ext::prelude::*;
+    /// let contents = b"hello world\n";
+    /// somedir.atomic_replace_with("somefilename", |f| -> io::Result<_> {
+    ///     f.write_all(contents)?;
+    ///     f.flush()?;
+    ///     use std::os::unix::prelude::PermissionsExt;
+    ///     let perms = cap_std::fs::Permissions::from_mode(0o600);
+    ///     f.get_mut().as_file_mut().set_permissions(perms)?;
+    ///     Ok(())
+    /// })
+    /// # }
+    /// ```
+    ///
+    /// Any existing file will be replaced.
+    fn atomic_replace_with<F, T, E>(
         &self,
         destname: impl AsRef<Path>,
         f: F,
     ) -> std::result::Result<T, E>
     where
-        F: FnOnce(&mut std::io::BufWriter<LinkableTempfile>) -> std::result::Result<T, E>,
+        F: FnOnce(&mut std::io::BufWriter<cap_tempfile::TempFile>) -> std::result::Result<T, E>,
         E: From<std::io::Error>;
 
-    /// Atomically write a file using specified permissions, replacing an existing one (if present).
-    ///
-    /// This wraps [`Self::new_linkable_file`] and [`crate::tempfile::LinkableTempfile::replace_with_perms`].
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn replace_file_with_perms<F, T, E>(
-        &self,
-        destname: impl AsRef<Path>,
-        perms: cap_std::fs::Permissions,
-        f: F,
-    ) -> std::result::Result<T, E>
-    where
-        F: FnOnce(&mut std::io::BufWriter<LinkableTempfile>) -> std::result::Result<T, E>,
-        E: From<std::io::Error>;
+    /// Atomically write the provided contents to a file.
+    fn atomic_write(&self, destname: impl AsRef<Path>, contents: impl AsRef<[u8]>) -> Result<()>;
 
-    /// Atomically write a file contents using specified permissions, replacing an existing one (if present).
-    ///
-    /// This wraps [`Self::new_linkable_file`] and [`crate::tempfile::LinkableTempfile::replace_with_perms`].
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn replace_contents_with_perms(
+    /// Atomically write the provided contents to a file, using specified permissions.
+    fn atomic_write_with_perms(
         &self,
         destname: impl AsRef<Path>,
         contents: impl AsRef<[u8]>,
@@ -94,6 +96,44 @@ fn map_optional<R>(r: Result<R>) -> Result<Option<R>> {
     }
 }
 
+enum DirOwnedOrBorrowed<'d> {
+    Owned(Dir),
+    Borrowed(&'d Dir),
+}
+
+impl<'d> Deref for DirOwnedOrBorrowed<'d> {
+    type Target = Dir;
+
+    fn deref(&self) -> &Self::Target {
+        match self {
+            Self::Owned(d) => d,
+            Self::Borrowed(d) => d,
+        }
+    }
+}
+
+/// Given a directory reference and a path, if the path includes a subdirectory (e.g. on Unix has a `/`)
+/// then open up the target directory, and return the file name.
+///
+/// Otherwise, reborrow the directory and return the file name.
+///
+/// It is an error if the target path does not name a file.
+fn subdir_of<'d, 'p>(d: &'d Dir, p: &'p Path) -> io::Result<(DirOwnedOrBorrowed<'d>, &'p OsStr)> {
+    let name = p
+        .file_name()
+        .ok_or_else(|| std::io::Error::new(std::io::ErrorKind::InvalidInput, "Not a file name"))?;
+    let r = if let Some(subdir) = p
+        .parent()
+        .filter(|v| !v.as_os_str().is_empty())
+        .map(|p| d.open_dir(p))
+    {
+        DirOwnedOrBorrowed::Owned(subdir?)
+    } else {
+        DirOwnedOrBorrowed::Borrowed(d)
+    };
+    Ok((r, name))
+}
+
 impl CapStdExtDirExt for Dir {
     fn open_optional(&self, path: impl AsRef<Path>) -> Result<Option<File>> {
         map_optional(self.open(path.as_ref()))
@@ -139,60 +179,50 @@ impl CapStdExtDirExt for Dir {
         }
     }
 
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn new_linkable_file<'p, 'd>(
-        &'d self,
-        target: &'p Path,
-    ) -> Result<crate::tempfile::LinkableTempfile<'p, 'd>> {
-        crate::tempfile::LinkableTempfile::new_in(self, target)
-    }
-
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn replace_file_with<F, T, E>(
+    fn atomic_replace_with<F, T, E>(
         &self,
         destname: impl AsRef<Path>,
         f: F,
     ) -> std::result::Result<T, E>
     where
-        F: FnOnce(&mut std::io::BufWriter<LinkableTempfile>) -> std::result::Result<T, E>,
+        F: FnOnce(&mut std::io::BufWriter<cap_tempfile::TempFile>) -> std::result::Result<T, E>,
         E: From<std::io::Error>,
     {
-        let t = self.new_linkable_file(destname.as_ref())?;
+        let destname = destname.as_ref();
+        let (d, name) = subdir_of(self, destname)?;
+        let t = cap_tempfile::TempFile::new(&d)?;
         let mut bufw = std::io::BufWriter::new(t);
         let r = f(&mut bufw)?;
         bufw.into_inner()
             .map_err(From::from)
-            .and_then(|t| t.replace())?;
+            .and_then(|t| t.replace(name))?;
         Ok(r)
     }
 
-    #[cfg(any(target_os = "android", target_os = "linux"))]
-    fn replace_file_with_perms<F, T, E>(
-        &self,
-        destname: impl AsRef<Path>,
-        perms: cap_std::fs::Permissions,
-        f: F,
-    ) -> std::result::Result<T, E>
-    where
-        F: FnOnce(&mut std::io::BufWriter<LinkableTempfile>) -> std::result::Result<T, E>,
-        E: From<std::io::Error>,
-    {
-        let t = self.new_linkable_file(destname.as_ref())?;
-        let mut bufw = std::io::BufWriter::new(t);
-        let r = f(&mut bufw)?;
-        bufw.into_inner()
-            .map_err(From::from)
-            .and_then(|t| t.replace_with_perms(perms))?;
-        Ok(r)
+    fn atomic_write(&self, destname: impl AsRef<Path>, contents: impl AsRef<[u8]>) -> Result<()> {
+        self.atomic_replace_with(destname, |f| f.write_all(contents.as_ref()))
     }
 
-    fn replace_contents_with_perms(
+    fn atomic_write_with_perms(
         &self,
         destname: impl AsRef<Path>,
         contents: impl AsRef<[u8]>,
         perms: cap_std::fs::Permissions,
     ) -> Result<()> {
-        let t = self.new_linkable_file(destname.as_ref())?;
-        t.replace_contents_using_perms(contents, perms)
+        self.atomic_replace_with(destname, |f| -> io::Result<_> {
+            // If the user is overriding the permissions, let's make the default be
+            // writable by us but not readable by anyone else, in case it has
+            // secret data.
+            #[cfg(unix)]
+            {
+                use std::os::unix::prelude::PermissionsExt;
+                let perms = cap_std::fs::Permissions::from_mode(0o600);
+                f.get_mut().as_file_mut().set_permissions(perms)?;
+            }
+            f.write_all(contents.as_ref())?;
+            f.flush()?;
+            f.get_mut().as_file_mut().set_permissions(perms)?;
+            Ok(())
+        })
     }
 }
